/*! app.js v0.0.7 | https://www.npmjs.com/package/app-methods */
(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.app = global.app || {})));
}(this, (function (exports) { 'use strict';

/* global App:false */
/**
 * @file 「Ping」的方法
 */
function hideBar$1 () {
  try {
    App.hideBar();
  } catch (e) {}
}

function showBar$1 () {
  try {
    App.showBar();
  } catch (e) {}
}

function exit$1 () {
  try {
    App.close();
  } catch (e) {}
}


var ping = Object.freeze({
	hideBar: hideBar$1,
	showBar: showBar$1,
	exit: exit$1
});

/* eslint-disable */
if (typeof weex === 'object') {
  window = {
    navigator: {
      userAgent: weex.config.env.platform + ' Needle/Weex'
    }
  };
}

/**
 * @namespace needle
 * @version 0.3.0
 * @global
 * @author hongju.wang@ele.me, jianxiang.jin@ele.me
 */
var callQueue = [];
var eventMap = {};
var stashedEvents = [];

/**
 * 判断是否是Android平台
 * @return {Boolean}
 * @function
 * @memberof needle
 * @return {Boolean}
 */
function isAndroid() {
  var ua = window.navigator.userAgent.toLowerCase();
  return ua.indexOf('android') > -1
}

/**
 * 判断是否是在 Needle 的WebView 环境
 * @return {Boolean}
 * @function
 * @memberof needle
 * @return {Boolean}
 */
function isNeedle() {
  var ua = window.navigator.userAgent.toLowerCase();
  return ua.indexOf('needle') > -1
}

/**
 * 判断是否是iOS平台
 * @return {Boolean}
 * @function
 * @memberof needle
 * @return {Boolean}
 */
function isIOS() {
  var ua = window.navigator.userAgent.toLowerCase();
  return ua.indexOf('iphone') > -1
}

/**
 * 判断是否是 Weex 环境
 * @return {Boolean}
 * @function
 * @memberof needle
 * @return {Boolean}
 */
function isWeex() {
  return typeof weex === 'object'
}

if (isWeex()) {
  var globalEvent = weex.requireModule('globalEvent');
  globalEvent.addEventListener('_needleEvent', function(event) {
    var eventName = event.eventName;
    var eventData = event.data;
    if (typeof event.data === 'object') {
      eventData = event.data;
    } else {
      try {
        eventData = JSON.parse(event.data);
      } catch (e) {}
    }
    var callback = eventAndCallbackMap[eventName];
    setTimeout(function() {
      !!callback && callback(eventData);
    }, 0);
  });

  globalEvent.addEventListener('_needleCall', function(event) {
    var response = event.data;
    if (typeof event.data === 'object') {
      response = event.data;
    } else {
      try {
        response = JSON.parse(event.data);
      } catch (e) {}
    }
    var callback = callingAndCallbackMap[response.id];
    response = response.result;
    setTimeout(function() {
      if (!!callback && !!response) {
        if (!!response.err) {
          callback(response.err, null);
        } else {
          callback(null, response.res);
        }
      }
      delete callingAndCallbackMap[response.id];
    }, 0);
  });
}

// For Android only
var uniqueId = 0; //请求调用的 message ID
var callingAndCallbackMap = {}; //调用以及回调的对应关系
var eventAndCallbackMap = {}; //事件名字以及事件回调的对应关系
if (!isWeex()) {
  window.__needleBrowserTunnel = {
    /**
     * read data from Android
     * {id, params, result}
     */
    read: function(responseString) {
      var response = JSON.parse(responseString);
      var callback = callingAndCallbackMap[response.id];
      response = response.result;
      setTimeout(function() {
        if (!!callback && !!response) {
          if (!!response.err) {
            callback(response.err, null);
          } else {
            callback(null, response.res);
          }
        }
        delete callingAndCallbackMap[response.id];
      }, 0);
    },
    on: function(eventName, data) {
      /**
       * receive event and event data from Android
       */
      var eventData = data;
      try {
        eventData = JSON.parse(data);
      } catch (e) {}
      var callback = eventAndCallbackMap[eventName];
      setTimeout(function() {
        !!callback && callback(eventData);
      }, 0);
    }
  };
}

/**
 * 根据插件名字调用(简称为名字调用)
 * @function
 * @memberof needle
 * @param {String} name 插件名字(自定义插件名字需要与客户端开发一起协商, 内置插件名字均已 needle 开头，业务自定义插件可添加自定义前缀避免命名冲突)
 * @param {Object} params 参数
 * @param {function} callback 回调函数, 函数签名callback(err, res),第一个总是 err, 第二个参数是正常 response（与 Node.js 回调类似）
 */
function execute(name, params, callback) {
  if (isWeex()) {
    var id = 'msg_' + uniqueId++ + '_' + new Date().getTime();
    callingAndCallbackMap[id] = callback;
    weex
      .requireModule('NeedleNativeModule')
      .execute({ id: id, name: name, params: params });
    return
  }

  if (isAndroid() && !!window.__needleClientTunnel) {
    var id = 'msg_' + uniqueId++ + '_' + new Date().getTime();
    callingAndCallbackMap[id] = callback;
    window.__needleClientTunnel.read(
      JSON.stringify({ id: id, name: name, params: params })
    );
  } else {
    if (!!window.WebViewJavascriptBridge) {
      _doCall(name, params, callback);
    } else {
      callQueue.push({ name: name, params: params, callback: callback });
    }
  }
}

/**
 * 判断是否 ready
 * @function
 * @memberof needle
 * @return {Boolean} true 表示 needle 初始化完毕
 */
function isReady() {
  return (
    !!window.WebViewJavascriptBridge ||
    !!window.__needleClientTunnel ||
    isWeex()
  )
}

function _doCall(name, params, callback) {
  !!window.WebViewJavascriptBridge &&
    window.WebViewJavascriptBridge.callHandler(name, params, function(res) {
      var response = isAndroid() ? JSON.parse(res) : res;
      if (!!callback && !!response) {
        if (!!response.err) {
          callback(response.err, null);
        } else {
          callback(null, response.res);
          if (window.__needle__debug) {
            console.log(
              'Plugin name:',
              name,
              ', Params:',
              params,
              ', Response:',
              response
            );
          }
        }
      }
    });
}

/**
 * 注册事件监听
 * @function
 * @memberof needle
 * @param {String} eventName  事件名字
 * @param {function} callback 事件发生时的回调
 */
function on(eventName, callback) {
  if (isWeex()) {
    eventAndCallbackMap[eventName] = callback;
    return
  }

  if (isAndroid() && !!window.__needleClientTunnel) {
    eventAndCallbackMap[eventName] = callback;
    return
  }

  eventMap[eventName] = { eventName: eventName, callback: callback };
  if (!!window.WebViewJavascriptBridge) {
    WebViewJavascriptBridge.registerHandler(eventName, function(data) {
      _doFlushEventListeners(eventName, data);
    });
  } else {
    stashedEvents.push({ eventName: eventName, callback: callback });
  }
}

function _doFlushEventListeners(eventName, data) {
  var event = eventMap[eventName];
  if (!!event) {
    event.callback(data);
  }
}

/**
 * 设备相关的 API
 *@namespace needle.app
 *@memberof needle
 */
var app = {
  /**
     * 获取设备信息
     * @example
     ## 返回值
     {
       appId : "me.ele.needle",
       appVersion : "1.0",
       buildNumber : 1,
       deviceId : "2146F58E-97A2-4D94-A18A-3C72B6C68BFB",
       platform : enum('Android', 'iOS')
       resolution : 640x1136,
       systemVersion : "10.1",
       networkType : enum('2G', '3G', '4G', 'WIFI', 'UNKNOW'),
       userAgent : "Mozilla/5.0 (iP......
       sdkVersion: '0.1.12'
      }
     * @function
     * @memberof needle.app
     * @param {function} callback 回调函数
     */
  getEnvInfo: function(callback) {
    execute('needle.app.env', {}, callback);
  },

  /**
   * 探测宿主手机某 APP 是否安装
   * @function
   * @memberof needle.app
   * @param {Object} param - {package: String, schemeUrl: String}, Android id 为 package，iOS 为 schemeUrl
   * @param {function} callback 回调函数 - res: {isAppInstalled: Boolean}
   */
  isAppInstalled: function(param, callback) {
    execute('needle.app.installation', param, callback);
  },

  /**
     * 编辑短信到发送短信页面等待用户确认发送
     * @example
     ##调用
     var option = {
            tel: 13857702077,
            message: 'this is a needle sms message.'
        };
     needle.app.sendSMS(option,function(err, res){
        if(!err){
           ;
        }
      });
     * @function
     * @memberof needle.app
     * @param {Object} param - {tel: String, message: String}
     * @param {function} callback 回调函数
     */
  sendSMS: function(param, callback) {
    execute('needle.app.sms', param, callback);
  },

  /**
     * 获取定位信息
     * @example
     * needle.app.getLocation(function(err, res){
     *   if(!err){
     *      res = {
                "longitude":89.33 ,
                "latitude": 101.11
            }
     *   }
     * });
     * @example
     ##成功响应
     {
       "longitude":89.33 ,
       "latitude": 101.11
     }
     * @function
     * @memberof needle.app
     * @param {function} callback 回调函数
     */
  getLocation: function(callback) {
    execute('needle.app.location', {}, callback);
  },

  /**
   * 在新的 WebView 实例中打开 Uri
   * @function
   * @example
   * //参数
   * {
   *   url: "http://www.google.com"
   *   external: true
   * }
   * @memberof needle.app
   * @param {Object} param {url: String, external: Boolean}, external 是否使用系统浏览器打开
   * @param {function} callback 回调函数
   */
  openUri: function(param, callback) {
    execute('needle.app.uri', param, callback);
  },

  /**
   * 清除webview缓存
   * Added: Android: 0.1.2
   * Added: iOS: 0.1.0
   * @function
   * @memberof needle.app
   * @param {function} callback 回调函数
   */
  clearWebsiteCache: function(callback) {
    execute('needle.app.web.cache.clear', {}, callback);
  }
};

/**
 * WebView 以及所在容器的一些相关操作
 *@namespace needle.container
 *@memberof needle
 */
var container = {
  /**
     * 显示 Toast
     * @example
     ## 调用
     {
       message: msg,
       duration: 1000,
       offset: 0
     }
     * @function
     * @memberof needle.container
     */
  toast: function(param) {
    execute('needle.container.toast', param, null);
  },

  /**
   * 关闭当前页面（Native 页面）
   * @function
   * @memberof needle.container
   */
  close: function() {
    execute('needle.container.window', { type: 'close' }, null);
  },

  /**
     * reload 网页，可以指定 url
     * @function
     * @example
     ## param 示例
     {
       url: 'http://www.google.com'
     }
     * @memberof needle.container
     * @param {function} callback 回调函数
     */
  reload: function(param, callback) {
    var param = param || {};
    param.type = 'reload';
    execute('needle.container.window', param, callback);
  },

  /**
   * 当前网页回退
   * @function
   * @memberof needle.container
   * @param {function} callback 回调函数
   */
  goBack: function(callback) {
    execute('needle.container.window', { type: 'back' }, callback);
  },

  /**
   * 显示 loading
   * @function
   * @memberof needle.container
   */
  showLoading: function() {
    execute('needle.container.loading', { type: 'show' }, null);
  },

  /**
   * 隐藏 loading
   * @function
   * @memberof needle.container
   */
  hideLoading: function() {
    execute('needle.container.loading', { type: 'hide' }, null);
  },

  /**
     * 设置导航栏的标题
     * @function
     * @example
     ## param 示例
     {
       title: ‘导航栏标题'
     }
     * @memberof needle.container
     * @param param 见示例
     * @param {function} callback 回调函数
     */
  setTitle: function(param, callback) {
    var param = param || {};
    param.type = 'set';
    execute('needle.container.title', param, null);
  },

  /**
     * 获取导航栏的标题
     * @function
     * @example
     ## callback res
     "{
       title: '获取到的标题'
     "}
     * @memberof needle.container
     * @param {function} callback 回调函数
     */
  getTitle: function(callback) {
    execute('needle.container.title', { type: 'get' }, callback);
  },

  /**
   * 显示原生导航栏
   * @function
   * @memberof needle.container
   * @param {function} callback 回调函数
   */
  showTitleBar: function(callback) {
    execute('needle.container.titleBar', { type: 'show' }, callback);
  },

  /**
   * 隐藏原生导航栏
   * @function
   * @memberof needle.container
   * @param {function} callback 回调函数
   */
  hideTitleBar: function(callback) {
    execute('needle.container.titleBar', { type: 'hide' }, callback);
  },

  /**
     * 设置原生导航栏右上角 menu 的icon
     * @function
     * @example
     ## param 示例
     {
       icon: 'http://icon_url' //
     }
     @example
     ## 调用示例
     needle.container.registerMenu({ icon: 'http://avatar.csdn.net/7/A/A/1_qq_19711823.jpg'}, function () {
            toast("Menu clicked:");
     });
     * @memberof needle.container
     * @param {Object} param {icon: String}
     * @param {function} onclick menu 点击会触发此 callback
     */
  registerMenu: function(param, onclick) {
    execute('needle.container.menu', param, null);
    on('needle.container.menu.onclick', onclick);
  },

  /**
     * 设置底部弹出 menu 的 icon，文字, key
     * @function
     * @example
     ## param 示例
     menus: [{
                        icon: '',
                        text: '',
                        key: '' // 唯一标识
                    }]
     ## callback res 示例
     {
       icon: '',
       text: '',
       key: ''
     }
     * @memberof needle.container
     * @param {Object} param - 见示例
     * @param {function} callback 回调函数
     */
  registerBottomMenu: function(param, callback) {
    execute('needle.container.bottomMenu', param, null);
    on('needle.container.bottomMenu.onclick', callback);
  }
};

/**
 * 图片选择以及图片预览相关 API
 *@namespace needle.image
 *@memberof needle
 */
var image = {
  /**
     * 选择图片
     * @example
     ## param 示例
     {
      type: enum('camera', 'album'),
      options: {
        maxSize: Number, // b, 比如 1024*50 = 50 Kb
        width: Number,
        height:Number
      }
     }
     @example
     ## callback res 示例
     {
       base64: 'base64 encode'
     }
     * @function
     * @memberof needle.image
     * @param {Object} param 见示例
     * @param {function} callback 回调函数
     */
  pick: function(param, callback) {
    execute('needle.image.picker', param, callback);
  },

  /**
     * 预览图片
     * @function
     @example
     ## param 示例
     { images :[{uri: 'uri'}] }
     * @memberof needle.image
     * @param {Object} param { images :[{uri: 'url'}] }
     * @param {function} callback 回调函数
     */
  preview: function(param, callback) {
    execute('needle.image.previewer', param, callback);
  },

  /**
     * 保存图片到系统相册
     * Added: Android 0.1.5
     * Added: iOS ?
     * @function
     @example
     ## param 示例
     { base64: 'base64 string'}
     * @memberof needle.image
     * @param {Object} param { base64: String}
     * @param {function} callback 回调函数
     */
  save: function(param, callback) {
    execute('needle.image.save', param, callback);
  }
};

/**
 * 可多 WebView 共享的 K-V 存储 API）
 * 注：不同 HOST 之间无法共享 Cache。例如 needle.ele.me 和 needle.elenet.me 无法共享 Cache。
 *@namespace needle.cache
 *@memberof needle
 */
var cache = {
  /**
     * 设置 K-V, 相同的 key 会被覆盖
     * @function
     * @example
     ## param 示例
     {key: 'key', value: 'value'}
     * @memberof needle.cache
     * @param {Object} param {key: String, value: String}
     * @param {function} callback 回调函数
     */
  set: function(param, callback) {
    var param = param || {};
    param.method = 'set';
    execute('needle.cache', param, callback);
  },

  /**
     * 获取 key 对应值
     * @function
     * @example
     ## param 示例
     {key: 'key'}
     * @memberof needle.cache
     * @param {Object} param {key: String}
     * @param {function} callback 回调函数
     */
  get: function(param, callback) {
    var param = param || {};
    param.method = 'get';
    execute('needle.cache', param, callback);
  },

  /**
     * delete key 对应值
     * Added: Android 0.1.3
     * Added: iOS: 0.1.0
     * @function
     * @example
     ## param 示例
     {key: 'key'}
     * @memberof needle.cache
     * @param {Object} param {key: String}
     * @param {function} callback 回调函数
     */
  delete: function(param, callback) {
    var param = param || {};
    param.method = 'delete';
    execute('needle.cache', param, callback);
  },

  /**
   * 删除所有 key 对应的值
   * Added: Android 0.1.3
   * Added: iOS: 0.1.0
   * @function
   * @memberof needle.cache
   * @param {function} callback 回调函数
   */
  deleteAll: function(callback) {
    var param = param || {};
    param.method = 'clear';
    execute('needle.cache', param, callback);
  }
};

if (!isWeex()) {
  if (window.WebViewJavascriptBridge) {
    initJS();
  } else {
    // WebViewJavascriptBridgeReady 事件由宿主 WebView 注入的一段 JavaScript 触发
    document.addEventListener('WebViewJavascriptBridgeReady', initJS, false);
  }
}

function _flushQueue() {
  callQueue.forEach(function(item) {
    _doCall(item.name, item.params, item.callback);
  });
  callQueue = [];
}

function _flushEventListener() {
  stashedEvents.forEach(function(event) {
    !!WebViewJavascriptBridge &&
      WebViewJavascriptBridge.registerHandler(event.eventName, function(data) {
        _doFlushEventListeners(event.eventName, data);
      });
  });
  stashedEvents = [];
}

function initJS() {
  _flushEventListener();
  _flushQueue();
  var needleReadyEvent = document.createEvent('Events');
  needleReadyEvent.initEvent('needleReady');
  document.dispatchEvent(needleReadyEvent);
}

var needle = {
  version: '0.3.0',
  isAndroid: isAndroid,
  isIOS: isIOS,
  isNeedle: isNeedle,
  isWeex: isWeex,
  isReady: isReady,
  execute: execute,
  on: on,
  app: app,
  container: container,
  image: image,
  cache: cache
};

/**
 * @file 「轩辕」的方法
 */

function exit$2 () {
  needle.container.close();
}

function hideBar$2 () {
  needle.container.hideTitleBar();
}

function showBar$2 () {
  needle.container.showTitleBar();
}

function getToken$1 () {
  return new Promise(function (resolve, reject) {
    needle.execute('xy.auth', {}, function (err, res) {
      if (err) {
        reject(err);
      } else {
        resolve(res.token);
      }
    });
  })
}


var xy = Object.freeze({
	exit: exit$2,
	hideBar: hideBar$2,
	showBar: showBar$2,
	getToken: getToken$1
});

var noop = function () {};

var dd = window.dd;
var exit$3;

if (dd) {
  var promise = new Promise(function (resolve) {
    dd.ready(resolve);
  });
  exit$3 = function () {
    promise.then(function () {
      dd.biz.navigation.close();
    });
  };
} else {
  exit$3 = noop;
}




var ding = Object.freeze({
	get exit () { return exit$3; }
});



var platforms = Object.freeze({
	ping: ping,
	xy: xy,
	ding: ding
});

var ua = navigator.userAgent.toLowerCase();

/**
 * @file 这个文件用于保存各种 APP 的 useragent 关键字
 */

var userAgents = {
  xy: 'eve',
  ping: 'qichat',
  ding: 'dingtalk'
};

var hasOwn = Object.prototype.hasOwnProperty;

var platform;

for (var key in userAgents) {
  if (hasOwn.call(userAgents, key)) {
    if (ua.indexOf(userAgents[key]) >= 0) {
      platform = key;
      break
    }
  }
}

// Ping 的 ua 是最近才加上的，
// 这里尝试看看有没有 Ping 的方法
if (!platform) {
  if (window.App && window.App.hideBar) {
    platform = 'ping';
  }
}

var platform$1 = platform || 'other';

var sdk = platforms[platform$1] || {};

// 由于 Ping 的 user-agent 没有唯一标志符，所以始终尝试调用一下 Ping 的 API
function hideBar () {
  hideBar$1();
  return sdk.hideBar && sdk.hideBar()
}

function showBar () {
  showBar$1();
  return sdk.showBar && sdk.showBar()
}

function exit () {
  exit$1();
  return sdk.exit && sdk.exit()
}

function getToken () {
  return sdk.getToken && sdk.getToken()
}

exports.platform = platform$1;
exports.hideBar = hideBar;
exports.showBar = showBar;
exports.exit = exit;
exports.getToken = getToken;

Object.defineProperty(exports, '__esModule', { value: true });

})));
